//-------------------------------------------------------------------
//	File:		YourCode.cpp
//	Comments:	Write your visual effect here.
//-------------------------------------------------------------------
#include "stdafx.h"
#include "Texture.h"

//#define WIREFRAME

//----------------------------------------
// Usefull debugger thingy.
void DebugOut(const char *szFormat, ...);
//----------------------------------------

// Change your App Name.
char szAppName[] = "Hufo's little tunnel";

// Timer info.  Very useful for keeping animated stuff in sync on different computers.
extern float fTime, fDeltaTime;

// Current window dimensions.
extern GLsizei g_w, g_h;
#ifndef NOEXTENSION
extern HWND g_hwnd;
#endif
#include "MyTunnel.h"

#ifndef NOEXTENSION
bool Splash()
{
	return MessageBox(NULL,
		"Warning: watching this effect too much can dramatically\n"
		"  harm your mental sanity. The author of this program\n"
		"  reject any responsability for it.\n"
		"  Watch it at your own risk :-)\n"
		"\n"
	  "Key usage:\n"
		" \"  \" : change tunnel shape\n"
		" \"+\"/\"-\" : increase/decrease speed\n"
		" \"*\"/\"/\" : change camera focal\n"
		" \"1\"..\"4\": change nb of poly\n"
		" ESC : stop it before being totally hypnotised\n"
		,szAppName
		,MB_OKCANCEL | MB_ICONWARNING) == IDOK;
}
#endif
// Use the demonstration texture.
CTexture gTexExample;
float x0; // window aspect ratio
#define XSTD (4.0/3.0)
float THole; // tunnel time
float TVit;

// Startup Stuff.
bool Init()	// Called right after the window is created, and OpenGL is initialized.
{
	// Reset the matrix to something we know.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	float x = (float)g_w/(float)g_h; // Correct the viewing ratio of the window in the X axis.
	if (x>XSTD)
		gluOrtho2D( -x, x, -1, 1 );	// Reset to a 2D screen space.
	else
		gluOrtho2D( -XSTD, XSTD, -XSTD/x, XSTD/x );	// Reset to a 2D screen space.
	x0=x;
	// Reset model view matrix stack.
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

#ifndef WIREFRAME
	gTexExample.LoadJPG( "Demo.jpg" );
	glEnable( GL_TEXTURE_2D );
	gTexExample.Use(); // Tells OpenGL to use this texture.
#endif

	glCullFace(GL_FRONT); // reject fliped faces
	glEnable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST); // no zbuffer

	THole=0.0;
	TVit=8000.0;
	HoleInit(); // initialise tunnel pos

	// If everything went well, then return true.
	// Returning false will cause the program to quit right away.
	return true;
}

void Shutdown() // Called right after the window is destroyed.
{
}

#ifndef NOEXTENSION
void UserKey(int k)
{
	switch (k)
	{
		case VK_SPACE:
			SinHole=!SinHole;
			break;
		case VK_ESCAPE:
			StopHole=True;
			break;
		case '+':
			TVit*=1.4141f;
			break;
		case '-':
			TVit/=1.4141f;
			break;
		case '*':
			HoleFoc*=1.1f;
			break;
		case '/':
			HoleFoc/=1.1f;
			break;
		case '4':
			HoledP=1;
			break;
		case '3':
			HoledP=2;
			break;
		case '2':
			HoledP=4;
			break;
		case '1':
			HoledP=8;
			break;
	}
}
#endif

// Draw all the scene related stuff.
void Render()
{
	float x = (float)g_w/(float)g_h; // Aspect Ratio
	if (x!=x0) // Window was resized and need reinit
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		if (x>XSTD)
			gluOrtho2D( -x, x, -1, 1 );	// Reset to a 2D screen space.
		else
			gluOrtho2D( -XSTD, XSTD, -XSTD/x, XSTD/x );	// Reset to a 2D screen space.
		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();
		x0=x;
	}

#ifndef NOEXTENSION
	static float secTime=fTime;
	static int nf=0;
	if (fTime-secTime>=1.0)
	{ // aff framerate
		char buf[256];
//		sprintf(buf,"%s - %g fps",szAppName,nf/(fTime-secTime));
		sprintf(buf,"%s - %g fps (%d tri)",szAppName,nf/(fTime-secTime),(HoleNbImgA-1)*HoleNbParImg*2/HoledP);
		SetWindowText(g_hwnd,buf);
		secTime=fTime;
		nf=0;
	}
#endif

	THole+=fDeltaTime*TVit;
	//CalcHole((fTime-t0)*4000); // animate the tunnel
	CalcHole((int)THole); // animate the tunnel
	if (HoleNbImgA==0)
	{
#ifndef NOEXTENSION
		if (StopHole)
			PostMessage(g_hwnd,WM_CLOSE,0,0);
#else
		glClear( GL_COLOR_BUFFER_BIT );
#endif
		return;
	}
	// and render it
	glClear( GL_COLOR_BUFFER_BIT );
	int p,i;
	float f1,f2;
	for (p=HoleNbImgA-2;p>=0;--p)
	{
		f1=min(1.0,1.0/(0.1+PtDist[p  ]*(0.25)));
		f2=min(1.0,1.0/(0.1+PtDist[p+1]*(0.25)));
#ifdef WIREFRAME
		glBegin( GL_LINES );
#else
		glBegin( GL_QUAD_STRIP );
#endif
			for (i=0;i<=HoleNbParImg;i+=HoledP)
			{
				glColor3f(f1,f1,f1);
				glTexCoord2f(Pt[p  ][i].u ,Pt[p  ][i].v );
				glVertex2f(  Pt[p  ][i].ex,Pt[p  ][i].ey);
				glColor3f(f2,f2,f2);
				glTexCoord2f(Pt[p+1][i].u ,Pt[p+1][i].v );
				glVertex2f(  Pt[p+1][i].ex,Pt[p+1][i].ey);
			}
		glEnd();
#ifdef WIREFRAME
		if (BBoxEmpty(&BBPlan[p]))
			glColor3f(1.0,0.0,0.0);
		else
			glColor3f(0.0,1.0,0.0);
		glBegin( GL_LINE_STRIP );
			glVertex2f(BBPlan[p].u0,BBPlan[p].v0);
			glVertex2f(BBPlan[p].u1,BBPlan[p].v0);
			glVertex2f(BBPlan[p].u1,BBPlan[p].v1);
			glVertex2f(BBPlan[p].u0,BBPlan[p].v1);
			glVertex2f(BBPlan[p].u0,BBPlan[p].v0);
		glEnd();
		glColor3f(f1,f1,f1);
		glBegin( GL_LINE_STRIP );
			for (i=0;i<=HoleNbParImg;i+=HoledP)
			{
				glTexCoord2f(Pt[p  ][i].u ,Pt[p  ][i].v );
				glVertex2f(  Pt[p  ][i].ex,Pt[p  ][i].ey);
			}
		glEnd();
#endif
	}
#ifndef NOEXTENSION
	++nf;
#endif
}
